@@ -1,6 +1,6 @@
 // Filename: load_store_unit.rs
-// Version:	 0.4
-// Date:	 31-12-2021 (DD-MM-YYYY)
+// Version:	 0.5
+// Date:	 25-01-2020 (DD-MM-YYYY)
 // Library:  gpcas_cpu_model
 //
 // Copyright (c) 2021 Kai Rese
@@ -36,4 +36,6 @@ pub struct LoadStoreUnit {
     pub store_count: usize,
     /// Connection to the provider of data memory.
     pub data_provider: MemoryConnection,
+    /// How many parallel ports there are to memory.
+    pub port_count: usize,
 }
