@@ -1,6 +1,6 @@
 // Filename: scheduler.rs
-// Version:	 0.3
-// Date:	 27-10-2021 (DD-MM-YYYY)
+// Version:	 0.4
+// Date:	 10-01-2022 (DD-MM-YYYY)
 //
 // Copyright (c) 2021 Kai Rese
 // This program is free software: you can redistribute it and/or modify
@@ -22,8 +22,6 @@ use serde::{Deserialize, Serialize};
 /// The properties of the register file.
 #[derive(Clone, Default, Deserialize, Serialize)]
 pub struct RegisterFile {
-    /// If the register file supports renaming.
-    pub renaming: bool,
     /// The number of physical general purpose registers.
     ///
     /// If not set, defaults to the number of architectural general purpose registers.
