@@ -1,6 +1,6 @@
 // Filename: v2.rs
-// Version:	 0.6
-// Date:	 30-12-2021 (DD-MM-YYYY)
+// Version:	 0.7
+// Date:	 10-01-2022 (DD-MM-YYYY)
 // Library:  gpcas_cpu_model
 //
 // Copyright (c) 2021 Kai Rese
@@ -24,6 +24,7 @@ use super::v3::{
     CPUModel as V3Model, CacheConfig as V3Cache, DecodeConfig as V3Decode, FetchConfig as V3Fetch,
     LoadStoreUnitConfig as V3LoadStoreUnit, PipelineConfig as V3Pipeline,
 };
+use super::v4::RegisterFile as V4RegisterFile;
 use crate::{config, CpuModel as NewModel, InstructionLatencies};
 
 use serde::{Deserialize, Serialize};
@@ -46,7 +47,7 @@ pub struct CPUModel {
     /// How many cycles the dispatch unit needs to dispatch an instruction.
     pub dispatch_cycle_count: usize,
     /// Configuration of the register file.
-    pub reg_file_config: config::RegisterFile,
+    pub reg_file_config: V4RegisterFile,
     /// Configuration of schedulers.
     pub schedulers: Vec<config::Scheduler>,
     /// Configuration for each pipeline from front end to the last stage.
