@@ -1,6 +1,6 @@
 // Filename: v3.rs
-// Version:	 0.3
-// Date:	 01-01-2021 (DD-MM-YYYY)
+// Version:	 0.5
+// Date:	 25-01-2022 (DD-MM-YYYY)
 // Library:  gpcas_cpu_model
 //
 // Copyright (c) 2021 Kai Rese
@@ -20,6 +20,10 @@
 
 //! Data structures for file version 3.
 
+use super::v4::{
+    BranchPredictor as V4BranchPredictor, CpuModel as V4Model, Fetch as V4Fetch,
+    LoadStoreUnit as V4LoadStoreUnit, RegisterFile as V4RegisterFile,
+};
 use crate::{config, CpuModel as NewModel, InstructionLatencies};
 
 use serde::{Deserialize, Serialize};
@@ -44,7 +48,7 @@ pub struct CPUModel {
     /// How many cycles the dispatch unit needs to dispatch an instruction.
     pub dispatch_cycle_count: usize,
     /// Configuration of the register file.
-    pub reg_file_config: config::RegisterFile,
+    pub reg_file_config: V4RegisterFile,
     /// Configuration of schedulers.
     pub schedulers: Vec<config::Scheduler>,
     /// Configuration for each pipeline from just after the front end to the last stage.
@@ -243,7 +247,7 @@ impl From<CPUModel> for NewModel {
             })
             .collect();
 
-        Self {
+        V4Model {
             max_vector_size,
             execution_latencies,
             fetch: fetch_config.into(),
@@ -257,10 +261,11 @@ impl From<CPUModel> for NewModel {
             pipelines,
             load_store_unit: load_store_unit_config.into(),
         }
+        .into()
     }
 }
 
-impl From<FetchConfig> for config::Fetch {
+impl From<FetchConfig> for V4Fetch {
     fn from(old: FetchConfig) -> Self {
         let FetchConfig {
             cache_connection,
@@ -284,7 +289,7 @@ impl From<FetchConfig> for config::Fetch {
     }
 }
 
-impl From<BranchPredictor> for Option<config::BranchPredictor> {
+impl From<BranchPredictor> for Option<V4BranchPredictor> {
     fn from(old: BranchPredictor) -> Self {
         match old {
             BranchPredictor::None => None,
@@ -293,18 +298,16 @@ impl From<BranchPredictor> for Option<config::BranchPredictor> {
                 entries,
                 bimodal_prediction,
                 return_address_stack_size,
-            } => Some(config::BranchPredictor::BranchTargetBuffer {
+            } => Some(V4BranchPredictor::BranchTargetBuffer {
                 entries,
                 bimodal_prediction,
                 return_address_stack_size,
             }),
-            BranchPredictor::TaggedGeometric(tables) => {
-                Some(config::BranchPredictor::TaggedGeometric {
-                    tables,
-                    table_history_length_start: 2,
-                    table_history_length_factor: 2.0,
-                })
-            }
+            BranchPredictor::TaggedGeometric(tables) => Some(V4BranchPredictor::TaggedGeometric {
+                tables,
+                table_history_length_start: 2,
+                table_history_length_factor: 2.0,
+            }),
         }
     }
 }
@@ -359,7 +362,7 @@ impl From<DecodeConfig> for config::Decoder {
     }
 }
 
-impl From<LoadStoreUnitConfig> for config::LoadStoreUnit {
+impl From<LoadStoreUnitConfig> for V4LoadStoreUnit {
     fn from(old: LoadStoreUnitConfig) -> Self {
         let LoadStoreUnitConfig {
             load_count,
