@@ -1,6 +1,7 @@
 // Filename: lib.rs
-// Version:	 0.15
-// Date:	 01-01-2021 (DD-MM-YYYY)
+// Version:	 0.16
+// Date:	 01
+// 10-01-2022 (DD-MM-YYYY)
 // Library:  gpcas_cpu_model
 //
 // Copyright (c) 2021 Kai Rese
@@ -161,10 +162,11 @@ impl InstructionLatencies {
 
 impl GpcasFileStruct for CpuModel {
     const FILE_IDENTIFIER: &'static str = "gpcas::cpu_model";
-    const CURRENT_FILE_VERSION: usize = 4;
+    const CURRENT_FILE_VERSION: usize = 5;
     const COMPATIBLE_VERSIONS: &'static [(usize, DeserializeFunction<Self>)] = &[
         (2, deserialize_upgrade_from::<legacy::v2::CPUModel, Self>),
         (3, deserialize_upgrade_from::<legacy::v3::CPUModel, Self>),
+        (4, deserialize_upgrade_from::<legacy::v4::CpuModel, Self>),
     ];
 }
 
