@@ -1,6 +1,6 @@
 // Filename: validation.rs
-// Version:	 0.1
-// Date:	 28-12-2021 (DD-MM-YYYY)
+// Version:	 0.3
+// Date:	 25-01-2022 (DD-MM-YYYY)
 // Library:  gpcas_cpu_model
 //
 // Copyright (c) 2021 Kai Rese
@@ -68,6 +68,12 @@ pub enum ValidationError {
     /// The cache must have a latency of at least one clock cycle for synchronisation reasons.
     /// Contains ID of the offending cache.
     CacheZeroLatency(usize),
+    /// The cache needs as least one entry in the line buffer, else no lines can be requested from
+    /// the next layer. Contains ID of the offending cache.
+    CacheZeroLineBufferSize(usize),
+    /// The cache needs to allow at least one miss to be in transit, else it freezes on the first
+    /// miss. Contains ID of the offending cache.
+    CacheZeroMaxOutstandingMisses(usize),
     /// Excessive cache bandwidth, most probably because it wasn't put in as power-of-two. Contains
     /// ID of the offending cache.
     CacheTooHighBandwidth(usize),
@@ -177,19 +183,19 @@ pub enum BranchPredictionError {
     ReturnAddressStackZeroEntries,
     /// The branch target buffer doesn't hold any entries and thus doesn't function.
     BranchTargetBufferZeroEntries,
-    /// No table holds any history, which makes the predictor non-functioning.
-    TaggedGeometricHistoryStartZero,
+    /// A history length below 2 would mean no history tracking, defeating the purpose.
+    TaggedGeometricHistoryLowTooSmall,
     /// The history size must increase from one table to the next for the predictor to function.
-    /// The set factor is too small to achieve that.
+    /// The calculated factor is too small to achieve that.
     TaggedGeometricHistoryFactorTooSmall,
     /// The global history grows to excessive values.
     TaggedGeometricHistoryTooLong,
+    /// The history length of the table with the longest one is smaller than the history length of
+    /// the table with the smallest one.
+    TaggedGeometricHistoryHighLowerThanLow,
     /// The prediction table contains no value, making it useless. Contains index of the offending
     /// table.
     TaggedGeometricTableZeroEntries(usize),
-    /// The prediction table doesn't have an entry count that is a power of two. This is required
-    /// because how entries are indexed. Contains ID of the offending table.
-    TaggedGeometricTableEntriesNoPowerOfTwo(usize),
     /// The tag of entries inside the table is too small to be useful. Contains index of the
     /// offending table.
     TaggedGeometricTableTooSmallTag(usize),
@@ -203,7 +209,7 @@ pub enum BranchPredictionError {
 const MAX_GLOBAL_HISTORY_LENGTH: usize = 1 << 16;
 /// The minimum size of entries in a table of a tagged geometric branch predictor before it's
 /// considered too small.
-const MIN_TAG_SIZE: usize = 2;
+const MIN_TAG_SIZE: usize = 4;
 /// The maximum size of entries in a table of a tagged geometric branch predictor before it's
 /// considered too large.
 const MAX_TAG_SIZE: usize = 32;
@@ -211,8 +217,6 @@ const MAX_TAG_SIZE: usize = 32;
 const MAX_BANDWIDTH: u16 = 20;
 /// The maximum valid cache size. Represents 1^30 bytes -> 1GiB.
 const MAX_CACHE_SIZE: usize = 30;
-/// Flag to implement optional validation for vector features before the ISA can provide the flag.
-const NEEDS_VECTOR_SUPPORT: bool = true;
 
 /// Validates the correctness of a CPU model configuration and returns a list of errors on failure.
 pub fn validate(model: &CpuModel, isa: &Isa) -> Result<(), FailedValidation> {
@@ -242,7 +246,9 @@ pub fn validate(model: &CpuModel, isa: &Isa) -> Result<(), FailedValidation> {
         errors.push(ValidationError::InstructionZeroLatency);
     }
 
-    if NEEDS_VECTOR_SUPPORT & (model.max_vector_size == 0 || model.max_vector_size % 128 != 0) {
+    if (isa.vector_register_count > 0)
+        & (model.max_vector_size == 0 || model.max_vector_size % 128 != 0)
+    {
         errors.push(ValidationError::VectorSizeNotMultiple);
     }
 
@@ -356,11 +362,13 @@ fn validate_branch_predictor(
         }
         config::BranchPredictor::TaggedGeometric {
             tables,
-            table_history_length_start,
-            table_history_length_factor,
+            table_history_length_low,
+            table_history_length_high,
         } => {
-            let length_start = *table_history_length_start;
-            let length_factor = *table_history_length_factor;
+            let length_first = *table_history_length_low;
+            let length_factor = (*table_history_length_high as f64 - length_first as f64)
+                .powf(1.0 / tables.len() as f64);
+            let length_last = *table_history_length_high;
 
             for (i, table) in tables.iter().enumerate() {
                 if table.tag_size < MIN_TAG_SIZE {
@@ -371,22 +379,20 @@ fn validate_branch_predictor(
 
                 if table.entries == 0 {
                     errors.push(BranchPredictionError::TaggedGeometricTableZeroEntries(i));
-                } else if !table.entries.is_power_of_two() {
-                    errors.push(BranchPredictionError::TaggedGeometricTableEntriesNoPowerOfTwo(i));
                 }
             }
 
-            if length_start == 0 {
-                errors.push(BranchPredictionError::TaggedGeometricHistoryStartZero);
+            if length_first < 2 {
+                errors.push(BranchPredictionError::TaggedGeometricHistoryLowTooSmall);
             }
 
-            if ((length_start as f64 * length_factor + 0.5) as usize) < length_start + 1 {
+            if ((length_first as f64 * length_factor + 0.5) as usize) < length_first + 1 {
                 errors.push(BranchPredictionError::TaggedGeometricHistoryFactorTooSmall);
             }
 
-            if ((length_start as f64 * length_factor.powi(tables.len() as i32) + 0.5) as usize)
-                > MAX_GLOBAL_HISTORY_LENGTH
-            {
+            if length_last < length_first {
+                errors.push(BranchPredictionError::TaggedGeometricHistoryHighLowerThanLow);
+            } else if length_last > MAX_GLOBAL_HISTORY_LENGTH {
                 errors.push(BranchPredictionError::TaggedGeometricHistoryTooLong);
             }
         }
@@ -425,6 +431,14 @@ fn validate_caches(
             errors.push(ValidationError::CacheZeroLatency(cache.id));
         }
 
+        if cache.line_buffer_size == 0 {
+            errors.push(ValidationError::CacheZeroLineBufferSize(cache.id));
+        }
+
+        if cache.max_outstanding_misses == 0 {
+            errors.push(ValidationError::CacheZeroMaxOutstandingMisses(cache.id));
+        }
+
         if cache.bandwidth > MAX_BANDWIDTH {
             errors.push(ValidationError::CacheTooHighBandwidth(cache.id));
         }
@@ -565,13 +579,13 @@ fn validate_register_file(
     let mut errors = Vec::new();
 
     if let Some(registers) = register_file.general_purpose_register_count {
-        if registers == 0 || registers < isa.register_bank_size {
+        if registers == 0 || registers < isa.general_purpose_register_count {
             errors.push(ValidationError::RegisterFileTooLowGeneralPurposeRegisters);
         }
     }
 
     if let Some(registers) = register_file.vector_register_count {
-        if NEEDS_VECTOR_SUPPORT & (registers == 0 || registers < isa.register_bank_size) {
+        if (isa.vector_register_count > 0) & (registers < isa.vector_register_count) {
             errors.push(ValidationError::RegisterFileTooLowVectorRegisters);
         }
     }
@@ -611,7 +625,7 @@ fn validate_schedulers(
 fn validate_pipelines(
     pipelines: &[&config::Pipeline],
     schedulers: &[&config::Scheduler],
-    _isa: &Isa,
+    isa: &Isa,
     gross_pipeline_count: usize,
 ) -> Result<(), Vec<ValidationError>> {
     let mut errors = Vec::new();
@@ -652,7 +666,7 @@ fn validate_pipelines(
             }
         }
 
-        if pipeline.vector & NEEDS_VECTOR_SUPPORT {
+        if pipeline.vector & (isa.vector_register_count > 0) {
             vector_load |= pipeline.memory_load;
             vector_store |= pipeline.memory_store;
 
@@ -717,7 +731,7 @@ fn validate_pipelines(
         errors.push(ValidationError::PipelineMissingGeneralPurposeStore);
     }
 
-    if NEEDS_VECTOR_SUPPORT {
+    if isa.vector_register_count > 0 {
         if !float_add {
             errors.push(ValidationError::PipelineMissingFloatAdd);
         }
@@ -905,6 +919,22 @@ impl std::fmt::Display for ValidationError {
                     *id
                 )
             }
+            ValidationError::CacheZeroLineBufferSize(id) => {
+                write!(
+                    f,
+                    "The cache with ID {} has a line buffer size of zero and thus can't request \
+                    new memory lines",
+                    *id
+                )
+            }
+            ValidationError::CacheZeroMaxOutstandingMisses(id) => {
+                write!(
+                    f,
+                    "The cache with ID {} doesn't allow any outstanding misses, to misses can't be \
+                    processed",
+                    *id
+                )
+            }
             ValidationError::CacheTooHighBandwidth(id) => {
                 write!(
                     f,
@@ -1167,10 +1197,10 @@ impl std::fmt::Display for BranchPredictionError {
                     "The branch target buffer has zero entries and thus doesn't work"
                 )
             }
-            BranchPredictionError::TaggedGeometricHistoryStartZero => {
+            BranchPredictionError::TaggedGeometricHistoryLowTooSmall => {
                 write!(
                     f,
-                    "The geometric history length starts at 0, thus not using any history"
+                    "The geometric history length starts below 2, thus not using any history"
                 )
             }
             BranchPredictionError::TaggedGeometricHistoryFactorTooSmall => {
@@ -1187,12 +1217,15 @@ impl std::fmt::Display for BranchPredictionError {
                     tables or lowering the growth factor"
                 )
             }
+            BranchPredictionError::TaggedGeometricHistoryHighLowerThanLow => {
+                write!(
+                    f,
+                    "The highest history length is longer than the shortest one"
+                )
+            }
             BranchPredictionError::TaggedGeometricTableZeroEntries(id) => {
                 write!(f, "Table {} doesn't have any entries", *id)
             }
-            BranchPredictionError::TaggedGeometricTableEntriesNoPowerOfTwo(id) => {
-                write!(f, "Table {}'s entry count isn't a power of two", *id)
-            }
             BranchPredictionError::TaggedGeometricTableTooSmallTag(id) => {
                 write!(
                     f,
